const _        = require('lodash');
const fs       = require('fs');
const fse      = require('fs-extra');
const path     = require('path');
const archiver = require('archiver');
const unzip    = require('node-unzip-2');
const glob     = require('glob');
const child    = require('child_process');

const electron = require('electron');
const app      = electron.app;
const ipc      = electron.ipcMain;
const dialog   = electron.dialog;

const iolabWindow = require('./iolabWindow');

// Error with regular package pdftk on mac
// https://www.pdflabs.com/tools/pdftk-the-pdf-toolkit/pdftk_server-2.02-mac_osx-10.11-setup.pkg
const PDFMerge = require('pdf-merge');

// Solve issue with the path on mac
if (process.platform == 'darwin') {
    process.env.PATH += ':/usr/local/bin';
} 

//
// Resource Directories
//
const lessonDataDir = path.normalize(app.getPath('documents') + '/IOLab-WorkFiles/activities/');
const reportDataDir = path.normalize(app.getPath('documents') + '/IOLab-WorkFiles/reports/');
const importTempDir = path.normalize(app.getPath('documents') + '/IOLab-WorkFiles/reports/Import_temp_dir/');

const pdftk = path.join(__dirname, '..', '..', 'pdftk')

global.iolabDirectories = {
    lessons: lessonDataDir,
    reports: reportDataDir,
};

//
// Print slide to PDF
//
ipc.on('check-pdf-merge', function(event) {
    if (process.platform == 'win32') {
    return event.sender.send('check-pdf-merge-reply', {
        available: true
    });
    }
    let cmd = 'ls /usr/local/bin/pdftk'
    child.exec(cmd, (err, out) => {
        event.sender.send('check-pdf-merge-reply', {
            available: !(err || out == ''),
            err: err,
            out: out
        });
    });
});

ipc.on('print-slide-to-pdf', function(event, reportID, slideKey) {
    let options = {
        landscape: false
    };
    iolabWindow.window.webContents.printToPDF(options, (error, data) => {
        if (error) { throw error; }

        let reportPath = path.join(reportDataDir, reportID);
        if (!fs.existsSync(reportPath)) {
            fs.mkdirSync(reportPath);
        }  

        let pdfFilePath = path.join(reportPath, `${slideKey}.pdf`);
        fs.writeFile(pdfFilePath, data, (error) => {
            if (error) throw error;
            event.sender.send('print-slide-to-pdf-reply', { success: true });
        });
    });
});

ipc.on('merge-pdf', function(event, reportID, lessonName, userName) {

    let filename = lessonName.slice(0, -5).replace(/\ /g, '_') +  '_' +
        (userName || reportID).replace(/\ /g, '_') + '.pdf';

    let options = {
        defaultPath: path.join(app.getPath('documents'), filename),
        filters: [ { name: 'PDF', extensions: ['pdf']} ]
    };

    let callback = function (fileName) {

        if (!fileName) {
            event.sender.send('merge-pdf-reply', { success: null });
            return;
        }

        let reportPath = path.join(reportDataDir, reportID);

        if (!fs.existsSync(reportPath)) {
            event.sender.send('merge-pdf-reply', { 
                success: false, 
                error: 'The lab archive does not contain sub-metadata directory!' 
            });            
            return;
        }  

        let files = fs.readdirSync(reportPath);
        let slides = files.filter(f => f.match(/slide.*pdf/i))
            .sort()
            .map(f => path.join(reportPath, f));

        if (slides.length == 0) {
            event.sender.send('merge-pdf-reply', { 
                success: false, 
                error: 'The lab archive does not contain pdf of slides!' 
            });
            return;
        }

        if (slides.length == 1) {
            fse.copySync(slides[0], fileName);
            event.sender.send('merge-pdf-reply', { success: true });
            return;
        }

        let pdfTkOptions = {
            libPath: pdftk,
            output: fileName
        }

        PDFMerge(slides, pdfTkOptions)
            .then(() => {
                event.sender.send('merge-pdf-reply', { success: true });
            })
            .catch(err => {
                event.sender.send('merge-pdf-reply', { success: false, error: 'PDFMerge error ! ' + err.message });
            });
    };
    dialog.showSaveDialog(iolabWindow.window, options, callback);
});

// 
// Upload file
//
ipc.on('upload-file', function(event, reportID, questionID, currentFileName) {
    let options = {
        defaultPath: app.getPath('documents'),
        filters: [ 
            { name: 'png', extensions: ['png']},
            { name: 'jpg', extensions: ['jpg']} 
        ]
    };

    let callback = function (fileName) {
        if (!fileName) {
            event.sender.send('upload-file-reply', { success: null });
            return;
        }

        let reportDir = path.join(reportDataDir, reportID);
        let outFileName = questionID + '-' + path.basename(fileName[0]);
        let outFilePath = path.join(reportDir, outFileName);
        let currentFilePath = currentFileName ? path.join(reportDir, currentFileName) : '';

        if (!fs.existsSync(reportDir)) {
            fs.mkdirSync(reportDir);
        }

        if (fs.existsSync(currentFilePath)) {
            fs.unlink(currentFilePath);
        }

        fs.createReadStream(fileName[0])
            .pipe(fs.createWriteStream(outFilePath))            
            .on('error', function(err) {
                event.sender.send('upload-file-reply', { success: false, message: 'Error during image upload', err });
                console.log(err);
            })
            .on('close', function () {
                event.sender.send('upload-file-reply', { success: true, fileName: outFileName });
            });
    };

    dialog.showOpenDialog(iolabWindow.window, options, callback);
});

//
// Export report 
//
ipc.on('export-report', function(event, dataID, lessonName, userName) {

    let filename = lessonName.slice(0, -5).replace(/\ /g, '_') + '_' +
        (userName || dataID).replace(/\ /g, '_') + '.lab';

    let options = {
        defaultPath: path.join(app.getPath('documents'), filename),
        filters: [ { name: 'Lab file', extensions: ['lab']} ]
    };

    let callback = function (fileName) {
        if (!fileName) {
            event.sender.send('export-report-reply', { success: null });
            return;
        }

        let archive = archiver('zip');
        let output = fs.createWriteStream(fileName);

        output.on('close', function() {
            event.sender.send('export-report-reply', {
                success: true,
                dataID: dataID,
                size: archive.pointer()
            });
            console.log(archive.pointer() + ' total bytes');
            console.log('archiver has been finalized and the output file descriptor has closed.');
        });

        archive.on('error', function(err) {
            event.sender.send('export-report-reply', {
                error: 'Error during report export !'
            });
        });

        archive.pipe(output);

        let metaDataFile = dataID + '.bson';

        if (!fs.existsSync(path.join(reportDataDir, metaDataFile))) {
            event.sender.send('export-report-reply', {
                error: 'There is not bson file ' + path.join(reportDataDir, metaDataFile)
            });
        }

        if (!fs.existsSync(path.join(reportDataDir, dataID))) {
            archive
                .file(path.join(reportDataDir, metaDataFile), { name: metaDataFile })
                .finalize();
            return;
        }

        archive
            .file(path.join(reportDataDir, metaDataFile), { name: metaDataFile })
            .directory(path.join(reportDataDir, dataID), '/' + dataID)
            .finalize();
    };

    dialog.showSaveDialog(iolabWindow.window, options, callback);
});

//
// Import report 
//
ipc.on('import-report', function(event) {    
    let options = {
        defaultPath: app.getPath('documents'),
        filters: [ { name: 'Lab', extensions: ['lab']} ]
    };

    let callback = function(fileName) {
        if (!fileName) return;

        fse.removeSync(importTempDir);
        fs.mkdirSync(importTempDir);

        fs.createReadStream(fileName[0])
            .pipe(unzip.Extract({path: importTempDir}))
            .on('error', function(err) {
                throw err;
            })
            .on('close', function () {

                console.log('unzip files', fs.readdirSync(importTempDir));

                let bson,
                    bsonFile = _.find(fs.readdirSync(importTempDir), function (file) {
                        return file.indexOf('.bson') > -1;
                    });

                if (bsonFile) {
                    bson = fs.readFileSync(path.join(importTempDir, bsonFile));
                    event.sender.send('import-report-reply', { bson: bson });

                } else {
                    throw new Error('There is no bson file in the "lab" archive');
                }
            });
    };

    dialog.showOpenDialog(iolabWindow.window, options, callback);
});

ipc.on('confirm-import-report', function(event, ID) {
    if (!fs.existsSync(path.join(importTempDir, `${ID}.bson`))) {
        throw new Error('The report with id ' + ID + ' cannot be found.');
    }

    fse.copySync(importTempDir, reportDataDir);
    fse.removeSync(importTempDir);
    
    event.sender.send('confirm-import-report-reply');
});

ipc.on('copy-acquisition-to-report', function(event, acquisitionPath, acqID, reportID) {

    let bsonFile = path.join(acquisitionPath, `${acqID}.bson`),
        rawFile = path.join(acquisitionPath, `${acqID}.raw`),
        reportDir = path.join(reportDataDir, reportID);

    if (fs.existsSync(bsonFile) && fs.existsSync(rawFile)) {

        // Create report directory if not yet present
        if (!fs.existsSync(reportDir)) {
            fs.mkdirSync(reportDir);
        }

        let calibrationFiles = glob.sync(path.join(acquisitionPath, 'cal_*.bson'), {});

        // Copy the file ...
        try {
            fse.copySync(bsonFile, path.join(reportDir, `${acqID}.bson`));
            fse.copySync(rawFile, path.join(reportDir, `${acqID}.raw`));

            calibrationFiles.forEach(function(file) {
                let filename = path.basename(file);
                fse.copySync(file, path.join(reportDir, filename));
            });

        } catch (err) {
            event.sender.send('copy-acquisition-to-report-reply', {
                success: false,
                message: 'Error while copying acquisition data files to the user report directory'
            });
            console.log(err);
        }

        event.sender.send('copy-acquisition-to-report-reply', { success: true });
    }

    event.sender.send('copy-acquisition-to-report-reply', { success: false, message: 'No file found' });
});

ipc.on('import-lesson', function(event) {
    let options = {
        defaultPath: app.getPath('documents'),
        filters: [ { name: 'Lesson', extensions: ['zip']} ]
    };

    let importDir = path.join(lessonDataDir, `Import_${(new Date()).toISOString().replace(/\:/g, '_')}`);

    let callback = function(fileName) {
        if (!fileName) {
            event.sender.send('import-lesson-reply', { success: null });
            return;
        }

        fs.mkdirSync(importDir);

        fs.createReadStream(fileName[0])
            .pipe(unzip.Extract({path: importDir}))
            .on('error', function(err) {
                throw err;
            })
            .on('close', function () {
                event.sender.send('import-lesson-reply', { success: true, importdir: importDir });
            });
    };

    dialog.showOpenDialog(iolabWindow.window, options, callback);
});